#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .booleanattribute import BooleanAttribute
from .content import Content
from .guidattribute import GuidAttribute
from .idattribute import IdAttribute


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .component import Component
from .componentgroup import ComponentGroup
from .componentgroupref import ComponentGroupRef
from .componentref import ComponentRef
from .condition import Condition
from .createfolder import CreateFolder
from .custom import Custom
from .customaction import CustomAction
from .directory import Directory
from .directoryref import DirectoryRef
from .directorysearch import DirectorySearch
from .feature import Feature
from .file import File
from .filesearch import FileSearch
from .fragment import Fragment
from .installexecutesequence import InstallExecuteSequence
from .mediatemplate import MediaTemplate
from .package import Package
from .product import Product
from .property import Property
from .ui import UI
from .uiref import UIRef
from .wix import Wix
from .wixvariable import WixVariable


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .wxs import WXSDocument
from .basebuildprocess import BaseBuildProcess
from .languagecode import LanguageCode
from .simplewxs import SimpleWXS


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
import re
from .schema import Product
from ...xml import Document, Element, Path


#--------------------------------------------------------------------------------
# WindowsInstaller XML Schema 문서.
#--------------------------------------------------------------------------------
class WXSDocument:
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	__document: Document


	#--------------------------------------------------------------------------------
	# 문서 프로퍼티.
	#--------------------------------------------------------------------------------
	@property
	def Document(self) -> Document:
		return self.__document
	

	#--------------------------------------------------------------------------------
	# 네임스페이스 프로퍼티.
	#--------------------------------------------------------------------------------
	@property
	def Namespaces(self) -> Dict[str, str]:
		return self.__namespaces


	#--------------------------------------------------------------------------------
	# WXS XML 루트 요소 프로퍼티.
	#--------------------------------------------------------------------------------
	@property
	def RootElement(self) -> Element:
		element = Element.Create(self.Document.RootXMLElement)
		return element


	#--------------------------------------------------------------------------------
	# 초기화 라이프사이클 메서드.
	#--------------------------------------------------------------------------------
	def __init__(self, document: Document = None) -> None:
		self.__document = document
		self.__namespaces = dict()
		self.__namespaces["wix"] = "http://schemas.microsoft.com/wix/2006/wi"


	#--------------------------------------------------------------------------------
	# 불러오기.
	#--------------------------------------------------------------------------------
	def LoadFromWXSFile(self, wxsFilePath: str) -> bool:
		if not self.__document.LoadFromFile(wxsFilePath):
			return False
		return True


	#--------------------------------------------------------------------------------
	# 저장하기.
	#--------------------------------------------------------------------------------
	def SaveToWXSFile(self, wxsFilePath: str) -> bool:
		xmlString: str = self.Document.SaveToString()
		with builtins.open(wxsFilePath, mode = "wt", encoding = "utf-8") as outputFile:
			outputFile.write(xmlString)


	#--------------------------------------------------------------------------------
	# 생성.
	#--------------------------------------------------------------------------------
	@staticmethod
	def Create() -> WXSDocument:
		wix: Element = Element.Create("Wix")

		product: Element = Element.Create("Product")
		wix.AddChild(product)
		package: Element = Element.Create("Package")
		product.AddChild(package)
		property: Element = Element.Create("Property", { "Id": "WIXUI_INSTALLDIR", "Value": "AltavaMaxPluginDirectory" })
		product.AddChild(property)
		mediaTemplate: Element = Element.Create("MediaTemplate", { "EmbedCab": "yes" })
		product.AddChild(mediaTemplate)
		feature: Element = Element.Create("Feauture", { "Id": "MainComponents" })
		product.AddChild(feature)
		componentGroupRef: Element = Element.Create("ComponentGroupRef", { "Id": "MainComponents" })
		feature.AddChild(componentGroupRef)
		wixVariable: Element = Element.Create("WixVariable", { "Id": "WixUILicenseRtf", "Value": "" })
		product.AddChild(wixVariable)
		ui: Element = Element.Create("UI")
		product.AddChild(ui)
		uiRef: Element = Element.Create("UIRef", { "Id": "WixUI_InstallDir" })
		ui.AddChild(uiRef)


		fragment: Element = Element.Create("Fragment")
		wix.AddChild(fragment)
		directroy: Element = Element.Create("Directory", { "Id": "TARGETDIR", "Name": "SourceDir" })
		fragment.AddChild(directroy)
		componentGroup = Element.Create("ComponentGroup", { "Id": "MainComponents" })
		fragment.AddChild(componentGroup)

		document: Document = Document.Create(wix)
		wxsDocument: WXSDocument = WXSDocument(document)
		return wxsDocument
	

	#--------------------------------------------------------------------------------
	# 불러오기.
	#--------------------------------------------------------------------------------
	@staticmethod
	def LoadFromWXSFile(wxsFilePath: str) -> WXSDocument:
		wxsDocument = WXSDocument()
		if not wxsDocument.LoadFromWXSFile(wxsFilePath):
			raise Exception()
		return wxsDocument


	#--------------------------------------------------------------------------------
	# 프로덕트 설정.
	#--------------------------------------------------------------------------------
	def SetProduct(self, product: Product) -> None:
		productElement: Element = self.RootElement.Find(".//wix:Product", self.Namespaces)
		if productElement:
			productElement.AddOrSetAttribute("Id", product.Id)
			productElement.AddOrSetAttribute("Name", product.Name)
			productElement.AddOrSetAttribute("Manufacturer", product.Manufacturer)
			productElement.AddOrSetAttribute("UpgradeCode", product.UpgradeCode)
			productElement.AddOrSetAttribute("Version", product.Version)
			productElement.AddOrSetAttribute("Language", product.Language)
		else:
			raise Exception("[wix3msi] Not found Product")


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
from enum import Enum
import os
import re
import uuid
from xml.dom import minidom as Minidom
from xml.dom.minidom import Document as MinidomDocument
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from ..data.idattribute import IdAttribute


#--------------------------------------------------------------------------------
# 폴더생성 요소.
# - Component의 자식으로 들어갈 수 있으며, 지정한 디렉토리 요소의 경로에 디렉토리가 없으면 생성해준다.
#--------------------------------------------------------------------------------
class CreateFolder:
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	Directory: IdAttribute # 디렉토리 요소의 고유식별자.


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .filesystem import DirectoryNode, FileNode, Node, NodeManager, NodeType
from .wix3.wxs import BooleanAttribute, Component, ComponentGroup, ComponentGroupRef, ComponentRef, Condition, Content, CreateFolder, Custom, CustomAction,\
Directory, DirectoryRef, DirectorySearch, Feature, File, FileSearch, Fragment, GuidAttribute, IdAttribute, InstallExecuteSequence, Package, Product, Property,\
UI, UIRef, Wix, WixVariable
from .wix3 import BaseBuildProcess, LanguageCode, SimpleWXS, WXSDocument
from .commandmanager import CommandManager


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
import os
import setuptools


#--------------------------------------------------------------------------------
# 전역 상수 목록.
#--------------------------------------------------------------------------------
UTF8: str = "utf-8"
FILE_READTEXT: str = "rt"


#--------------------------------------------------------------------------------
# 참조 메타 데이터 목록.
#--------------------------------------------------------------------------------
NAME: str = "wix3msi"
AUTHOR: str = "ddukbaek2"
AUTHOR_EMAIL: str = "ddukbaek2@gmail.com"
DESCRIPTION: str = "Create robust and reliable Windows installers with ease. This library leverages Wix Toolset v3 to generate XML definitions, allowing developers to focus on installation logic instead of XML syntax."
LONG_DESCRIPTION_CONTENT_TYPE: str = "text/markdown"
URL: str = "https://ddukbaek2.com"
PYTHON_REQUIRES: str = ">=3.9.7" # 실제로는 3.10.11 이상.
LONGDESCRIPTION: str = str()
with open(file = "README.md", mode = FILE_READTEXT, encoding = UTF8) as file: LONGDESCRIPTION = file.read()
with open(file = "VERSION", mode = FILE_READTEXT, encoding = UTF8) as file: VERSION = file.read()


#--------------------------------------------------------------------------------
# 빌드.
#--------------------------------------------------------------------------------
setuptools.setup(
	name = NAME,
	version = VERSION,
	author = AUTHOR,
	author_email = AUTHOR_EMAIL,
	description = DESCRIPTION,
	long_description = LONGDESCRIPTION,
	long_description_content_type = LONG_DESCRIPTION_CONTENT_TYPE,
	url = URL,
	packages = setuptools.find_packages(where = "src"),
	include_package_data = True,
	package_dir = { "": "src" },
	package_data = {
		"": [
			"res/*"
		],
	},
	scripts = [

	],
	entry_points = {
		"console_scripts": [
			"wix3msi=wix3msi.commandmanager:CommandManager.Run"
		]
	},
	install_requires = [
		"xid-xpl"
	],
	classifiers = [
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: Microsoft :: Windows"
	],
	python_requires = PYTHON_REQUIRES
)